unit shmemev;
interface

uses
  Windows;

type

  THandledObject = class(TObject)
  protected
    FHandle: THandle;
  public
    destructor Destroy; override;
    property Handle: THandle read FHandle;
  end;

  TEvent = class(THandledObject)
  public
    constructor Create(const Name: string; Manual: Boolean);
    procedure Signal;
    procedure Reset;
    function Wait(TimeOut: Integer): Boolean;
  end;

  TSharedMem = class(THandledObject)
  private
    FName: string;
    FSize: Integer;
    FCreated: Boolean;
    FFileView: Pointer;
  public
    constructor Create(const Name: string; Size: Integer);
    destructor Destroy; override;
    property Name: string read FName;
    property Size: Integer read FSize;
    property Buffer: Pointer read FFileView;
    property Created: Boolean read FCreated;
  end;

const
  sharedlength = 1000;

var
  SharedMem : TSharedMem;
  Event : TEvent;

procedure writeshared(s : string);
procedure create_ipc;

implementation

//#############################################################################
procedure writeshared(s : string);
var
  i : integer;
  pstr : ^char;
begin
  for i:= length(s) to sharedlength do s:= s + char(#255);
  pstr:= SharedMem.buffer;
  for i:= 1 to length(s) do begin
    pstr^:= s[i];
    inc(pstr);
  end;
  Event.Signal;
  sleep(30);
end;

procedure create_ipc;
begin
  Event:= TEvent.Create('debugev', false);
  SharedMem:= TSharedMem.create('debugsm', sharedlength);
end;

//#############################################################################
destructor THandledObject.Destroy;
begin
  if FHandle <> 0 then
    CloseHandle(FHandle);
end;

//#############################################################################
constructor TEvent.Create(const Name: string; Manual: Boolean);
begin
  FHandle := CreateEvent(nil, Manual, False, PChar(Name));
  if FHandle = 0 then halt;
end;

procedure TEvent.Reset;
begin
  ResetEvent(FHandle);
end;

procedure TEvent.Signal;
begin
  SetEvent(FHandle);
end;

function TEvent.Wait(TimeOut: Integer): Boolean;
begin
  Result := WaitForSingleObject(FHandle, TimeOut) = WAIT_OBJECT_0;
end;

//#############################################################################
constructor TSharedMem.Create(const Name: string; Size: Integer);
begin
  try
    FName := Name;
    FSize := Size;
    FHandle := CreateFileMapping($FFFFFFFF, nil, PAGE_READWRITE, 0,
        Size, PChar(Name));
    if FHandle = 0 then halt;
    FCreated := GetLastError = 0;
    FFileView := MapViewOfFile(FHandle, FILE_MAP_WRITE, 0, 0, Size);
    if FFileView = nil then halt;
  except
    writeln('Error creating shared memory ', Name, GetLastError);
  end;
end;

destructor TSharedMem.Destroy;
begin
  if FFileView <> nil then
    UnmapViewOfFile(FFileView);
  inherited Destroy;
end;

end.
