{#############################################################################
#                _         _                  _____                          #
#               | |       | |                / __  \                         #
#      ___  ___ | | _____ | |__   __ _ _ __  `' / /'                         #
#     / __|/ _ \| |/ / _ \| '_ \ / _` | '_ \   / /                           #
#     \__ \ (_) |   < (_) | |_) | (_| | | | |./ /___                         #
#     |___/\___/|_|\_\___/|_.__/ \__,_|_| |_|\_____/                         #
#                                                                            #
#  Kohn's Sokoban2                                                           #
#                                                                            #
#  Copyright (C) 2001  Norman Krebs                                          #
#                                                                            #
#    This program is free software; you can redistribute it and/or           #
#    modify it under the terms of the GNU General Public License as          #
#    published by the Free Software Foundation; either version 2 of          #
#    the License, or (at your option) any later version.                     #
#                                                                            #
#    This program is distributed in the hope that it will be useful,         #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        #
#    General Public License for more details.                                #
#                                                                            #
#    You should have received a copy of the GNU General Public License       #
#    along with this program; if not, write to the Free Software             #
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.               #
#                                                                            #
#############################################################################}
program sokoban2;

uses
  Forms,
  registry,
  windows,
  sysutils,
  Unit1 in 'Unit1.pas' {Form1},
  Unit2 in 'Unit2.pas' {Form2},
  Unit3 in 'Unit3.pas' {Form3},
  Unit4 in 'Unit4.pas' {Form4},
  Unit5 in 'Unit5.pas' {Form5};

{$R *.RES}

var i : integer;
    ss : string;
    flag : boolean;
    lpbuffer : TMEMORYSTATUS;
    mem : cardinal;
    err : integer;
    ZEROSTR : string = '';

label LOADOK;

procedure AlignMenuRight(AbItem: Integer; Form: TForm);
var m:TMenuItemInfo;
     h:hMenu;
     s:shortstring;
begin
   h:=GetMenu(Form.Handle);
   m.cbSize:=SizeOf(m);
   m.cch:=SizeOf(s);
   m.dwTypeData:=pchar(@s[1]);
   m.fMask:=MIIM_TYPE;

   GetMenuItemInfo(h,AbItem,true,m);

   m.ftype:=m.ftype or MFT_RIGHTJUSTIFY;

   SetMenuItemInfo(h,AbItem,true,m);
end;

function getenvstr(sname : string) : string;
var d : pointer;
    s : string;
begin
  result:= 'ENVVAR_NOT_FOUND!';
  err:= 1;
  d:= GetEnvironmentStrings;
  repeat
    s:= pchar(d);
    if s <> ZEROSTR then begin
      if ( pos(uppercase(sname), uppercase(s)) = 1 ) then begin
        delete(s, 1, length(sname) + 1);
        result:= s;
        err:= 0;
        exit;
      end;
      pchar(d):= pchar(d) + 1 + Length(s);
    end;
  until s = ZEROSTR;
end;

begin
  Application.Initialize;
  Application.CreateForm(TForm1, Form1);
  Application.CreateForm(TForm2, Form2);
  Application.CreateForm(TForm3, Form3);
  Application.CreateForm(TForm4, Form4);
  Application.CreateForm(TForm5, Form5);
  form1.DoubleBuffered:= true;
  form1.panel1.DoubleBuffered:= true;
  outc(_idstr_);
  outc('this is the sokoban information console');

  getwindowsversion(winver, winsubver, BuildNr, w31, w95, wnt, w2k);

  globalmemorystatus(lpbuffer);
  outc('installed memory: ' + inttostr(lpbuffer.dwTotalPhys div 1024) + ' KB');
  outc('available memory: ' + inttostr(lpbuffer.dwAvailPhys div 1024) + ' KB');
  mem:= (lpbuffer.dwTotalPhys - lpbuffer.dwAvailPhys) div 1024;
  outc('used memory:      ' + inttostr(mem) + ' KB');

  if w31 then ;
  if w95 then outc('Windows Version ' + inttostr(winver) + '.' + inttostr(winsubver) + ' build: ' + inttostr(buildnr));
  if wnt then begin
    if winver = 4 then
      outc('Windows NT Version ' + inttostr(winver) + '.' + inttostr(winsubver) + ' build: ' + inttostr(buildnr))
    else
      outc('Windows 2000 Version ' + inttostr(winver) + '.' + inttostr(winsubver) + ' build: ' + inttostr(buildnr))
  end;

  outc('');
  outc('main() formcreation OK');

  form1.loadroomstomenu;
  form1.loadbmpstomenu;
  form1.loadbmps;

// bookmarks
  if w95 then begin
    bookmarkdir:= 'bookmarks\';
    createdir(workdir + bookmarkdir);
  end
  else begin
    bookmarkdir:= getenvstr('userprofile');
    if bookmarkdir[length(bookmarkdir)] <> '\' then bookmarkdir:= bookmarkdir + '\';
    bookmarkdir:= bookmarkdir + 'sokoban2bookm\';
    createdir(bookmarkdir);
  end;
  outc('bookmarkdir ' + bookmarkdir);

  form1.loadbookmarkstomenu;

  regfile:= TregIniFile.create('system');
  regfile.rootkey:= HKEY_CURRENT_USER;
  form1.left:= strtoint(regfile.ReadString('\Software\NKsoftware\sokoban2','left', '1'));
  form1.top:= strtoint(regfile.ReadString('\Software\NKsoftware\sokoban2','top', '1'));
  form1.Logging1.checked:= (regfile.ReadString('\Software\NKsoftware\sokoban2','Logging', '0') = '1');
  form1.SaveSteps1.checked:= (regfile.ReadString('\Software\NKsoftware\sokoban2','SaveSteps', '0') = '1');
  _OUT_SIZER:= strtofloat(regfile.ReadString('\Software\NKsoftware\sokoban2','size', '1'));
  form2.left:= 5;
  form2.top:= 5;
  form2.width:= screen.width div 3;
  form2.height:= screen.height div 3 * 2;
  ss:= regfile.ReadString('\Software\NKsoftware\sokoban2','room', form1.rooms1.items[0].caption);

  for i:= 0 to form1.rooms1.count - 1 do begin
    form1.rooms1.items[i].checked:= false;
    if ss = form1.rooms1.items[i].caption then form1.rooms1.items[i].checked:= true;
  end;

  flag:= false;
  for i:= 0 to form1.rooms1.count - 1 do
    if form1.rooms1.items[i].checked then flag:= true;
  if not flag then form1.rooms1.items[0].checked:= true;

  i:= strtoint(regfile.ReadString('\Software\NKsoftware\sokoban2','layout', '0'));
  if form1.layouts1.count < (i + 1) then i:= 0;
  form1.layouts1.items[i].checked:= true;

  for i:= 0 to form1.rooms1.Count - 1 do begin
    reached[i].a:= strtoint(regfile.ReadString('\Software\NKsoftware\sokoban2\rooms', form1.rooms1.items[i].caption, '0'));
    outc('main() ' + inttostr(i) + ' ' + form1.rooms1.items[i].caption + ' reached ' + inttostr(reached[i].a));
  end;

  form1.animation1.checked:= regfile.ReadString('\Software\NKsoftware\sokoban2', 'animation', '0') = '1';
  regfile.Free;
  form1.loadbmps;
  form1.loadrooms;

  for i:= 0 to form1.rooms1.count - 1 do
    if form1.rooms1.items[i].checked then begin
      roomnr:= reached[i].a + 1;
      if roomnr >= rct then roomnr:= rct;
      break;
    end;

  form1.loadsingleroom;
  infoout;
LOADOK:
  outc('main() ready');
  globalmemorystatus(lpbuffer);
  outc('used memory at start:       ' + inttostr(mem) + ' KB');
  outc('used memory after dataload: ' + inttostr((lpbuffer.dwTotalPhys - lpbuffer.dwAvailPhys) div 1024) + ' KB');
  outc('used memory by sokoban2:    ' + inttostr(((lpbuffer.dwTotalPhys - lpbuffer.dwAvailPhys) div 1024) - mem) + ' KB for data');
  form2.RichEdit1.Clear;
  AlignMenuRight(4, Form1);
  Application.Run;
end.
